import * as React from "react";
import styled from "@emotion/styled";
import { IconButton as MuiIconButton, TableRow } from "@material-ui/core";
import ChevronRightIcon from "@material-ui/icons/ChevronRight";
import KeyboardArrowDownIcon from "@material-ui/icons/KeyboardArrowDown";

import type { TableRowProps } from "./table";
import { TableCell } from "./table";

const IconButton = styled(MuiIconButton)({
  padding: "0",
  color: "#0D1030",
});

const ChevronRight = styled(ChevronRightIcon)(props => ({
  color: props?.["data-disabled"] ? "#E7E7EA" : "unset",
}));

export interface AccordionRowProps {
  headings: React.ReactElement[];
  children: React.ReactElement<TableRowProps> | React.ReactElement<TableRowProps>[];
}

export const AccordionRow = ({ headings, children }: AccordionRowProps) => {
  const [open, setOpen] = React.useState(false);
  const hasChildren = React.Children.count(children) !== 0;

  const onClick = () => {
    if (hasChildren) {
      setOpen(isOpen => !isOpen);
    }
  };
  return (
    <>
      <TableRow>
        {headings.map((heading: any, index: number) => {
          const icon = (
            <IconButton onClick={onClick}>
              {open ? <KeyboardArrowDownIcon /> : <ChevronRight data-disabled={!hasChildren} />}
            </IconButton>
          );
          return (
            <TableCell key={heading} data-border style={index === 0 ? { display: "flex" } : {}}>
              {index === 0 && icon}
              <div style={{ alignSelf: "center" }}>{heading}</div>
            </TableCell>
          );
        })}
      </TableRow>
      {open && children}
    </>
  );
};
